/***************************************************************************
*                 _                                _                       *
*                /_\       Chess Board Engine     /_\                      *
*                \_/      -------------------     \_/                      *
*                /_\        Based on Rules        /_\                      *
*                |_|      -------------------     |_|                      *
*               /___\      GNU GPL Version 2     /___\                     *
*                                                                          *
***************************************************************************/
/* Header Files */
#include "engine.h"

/* Constructor */
Game::Game() {}

/* Window */
static WINDOW *winF;
static WINDOW *winE;

//-----------------------------------------------------------------------------

/* Material */
int ebit[E] = {'1','0','0','0','1','0','1','1','1','0',0};
int sbit[E] = { 8, 5, 2, 4, 7, 0, 1, 3, 6,0};
int fbit[E] = { 1, 3, 1, 4, 7, 5, 2, 6, 0,0};

//-----------------------------------------------------------------------------

/* Circle */
int circle[H][A] = { 
	
{0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0},
{0x00,0xDF,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0x00,0},
{0x00,0xDE,0xC3,0xB0,0xB1,0xB2,0xB3,0xB4,0xCB,0x00,0},
{0x00,0xDD,0xC2,0xAF,0xA4,0xA5,0xA6,0xB5,0xCC,0x00,0},
{0x00,0xDC,0xC1,0xAE,0xA3,0xA0,0xA7,0xB6,0xCD,0x00,0},
{0x00,0xDB,0xC0,0xAD,0xA2,0xA1,0xA8,0xB7,0xCE,0x00,0},
{0x00,0xDA,0xBF,0xAC,0xAB,0xAA,0xA9,0xB8,0xCF,0x00,0},
{0x00,0xD9,0xBE,0xBD,0xBC,0xBB,0xBA,0xB9,0xD0,0x00,0},
{0x00,0xD8,0xD7,0xD6,0xD5,0xD4,0xD3,0xD2,0xD1,0x00,0},
{0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0}};

//-----------------------------------------------------------------------------

/* Chess Engine Array */
char cengine[H][A][E][C2] = {
			
{{{' ',9},{'8',9},{'7',9},{'6',9},{'5',9},{'4',9},{'3',9},{'2',9},{'1',9},{' ',9},{0,0}}},
{{{'a',9},{'R',5},{'P',2},{' ',9},{' ',9},{' ',9},{' ',9},{'p',2},{'r',5},{'a',9},{0,0}}},
{{{'b',9},{'N',3},{'P',2},{' ',9},{' ',9},{' ',9},{' ',9},{'p',2},{'n',3},{'b',9},{0,0}}},
{{{'c',9},{'B',4},{'P',2},{' ',9},{' ',9},{' ',9},{' ',9},{'p',2},{'b',4},{'c',9},{0,0}}},
{{{'d',9},{'Q',6},{'P',2},{' ',9},{' ',9},{' ',9},{' ',9},{'p',2},{'q',6},{'d',9},{0,0}}},
{{{'e',9},{'K',7},{'P',2},{' ',9},{' ',9},{' ',9},{' ',9},{'p',2},{'k',7},{'e',9},{0,0}}},
{{{'f',9},{'B',4},{'P',2},{' ',9},{' ',9},{' ',9},{' ',9},{'p',2},{'b',4},{'f',9},{0,0}}},
{{{'g',9},{'N',3},{'P',2},{' ',9},{' ',9},{' ',9},{' ',9},{'p',2},{'n',3},{'g',9},{0,0}}},
{{{'h',9},{'R',5},{'P',2},{' ',9},{' ',9},{' ',9},{' ',9},{'p',2},{'r',5},{'h',9},{0,0}}},
{{{' ',9},{'8',9},{'7',9},{'6',9},{'5',9},{'4',9},{'3',9},{'2',9},{'1',9},{' ',9},{0,0}}}};

//-----------------------------------------------------------------------------

/* Variables */
int A1, A2, O, a, b;

/* Formel */
void Game::formel(int n,int r)
{
	/* PI */
	float PI = 3.14159;
	
	/* Formel */
	a = (360 / n);
	b = (PI * r * a) / 180;
	A1 = (b * r) / 2;
	A2 = (PI * (r * r) * a) / 360;
	O = 4 * PI * (r * r);
		
	/* Control Radius */
	//mvwprintw(winE,L1,28,"a:%3d b:%3d A:%3d A:%3d O:%3d", a, b, A1, A2, O);
}

//-----------------------------------------------------------------------------

/* Engine */
void Game::engine(int *ES,int F1,int F2,int T1,int T2)
{
	/* Engine Time */
	unsigned int usecs = 200000;
		
	/* Calculation */
	char emove[20];
	int X = 0;
	int Y = 10;
	int S1 = 0;
	int S2 = 0;

	/* Tree */
	int M1, M2, M3, M4; 

	M1 = ES[1];
	M2 = ES[2];
	M3 = ES[3];
	M4 = ES[4];
	
	//----------------------------------------------------------------------

	/* a - h */
	emove[1+X] = 'a';
	emove[2+X] = 'b';
	emove[3+X] = 'c';
	emove[4+X] = 'd';
	emove[5+X] = 'e';
	emove[6+X] = 'f';
	emove[7+X] = 'g';
	emove[8+X] = 'h';
	
	/* 1 - 8 */
	emove[1+Y] = '1';
	emove[2+Y] = '2';
	emove[3+Y] = '3';
	emove[4+Y] = '4';
	emove[5+Y] = '5';
	emove[6+Y] = '6';
	emove[7+Y] = '7';
	emove[8+Y] = '8';
	
	//----------------------------------------------------------------------
	
	/* Ebene 1 */
	if((M1 == 0) || (M1 == 1))
	{
		/* Ebene 2  && Ebene 3 */
		if((M2 == 0) && (M3 == 0))
		{
			/* Move From */
			amove[0] = emove[F1+X];
			amove[1] = emove[F2+Y];
			
			/* Ebene 4 */
			if(M4 == 0)
			{
				/* Calculation */
				S1 = F1 + T1;
				S2 = F2 + T2;		
			}
			else if(M4 == 1)
			{
				/* Calculation */
				S1 = F1 + T1;
				S2 = F2 + T2;
			}
		}
		
		//------------------------------------------------------------------
		
		/* Ebene 2  && Ebene 3 */
		if((M2 == 0) && (M3 == 1))
		{
			/* Move From */
			amove[0] = emove[F1+X];
			amove[1] = emove[F2+Y];
			
			/* Ebene 4 */
			if(M4 == 0)
			{	
				/* Calculation */
				S1 = F1 + T1;
				S2 = F2 + T2;		
			}
			else if(M4 == 1)
			{			
				/* Calculation */
				S1 = F1 + T1;
				S2 = F2 + T2;	
			}
		}
		
		//------------------------------------------------------------------
		
		/* Ebene 2  && Ebene 3 */
		if((M2 == 1) && (M3 == 0))
		{
			/* Move From */
			amove[0] = emove[F1+X];
			amove[1] = emove[F2+Y];
			
			/* Ebene 4 */
			if(M4 == 0)
			{	
				/* Calculation */
				S1 = F1 + T1;
				S2 = F2 + T2;		
			}
			else if(M4 == 1)
			{				
				/* Calculation */
				S1 = F1 + T1;
				S2 = F2 + T2;		
			}
		}
		
		//------------------------------------------------------------------
		
		/* Ebene 2  && Ebene 3 */
		if((M2 == 1) && (M3 == 1))
		{
			/* Move From */
			amove[0] = emove[F1+X];
			amove[1] = emove[F2+Y];
			
			/* Ebene 4 */
			if(M4 == 0)
			{				
				/* Calculation */
				S1 = F1 + T1;
				S2 = F2 + T2;	
			}
			else if(M4 == 1)
			{				
				/* Calculation */
				S1 = F1 + T1;
				S2 = F2 + T2;	
			}
		}
	}
	
	//----------------------------------------------------------------------
	
	/* Direction */
	if(((S1 >= C1) && ( S1 <= C8)) && ((S2 >= C1) && (S2 <= C8)))
	{			
		/* Circle */
		if((circle[S1][S2] >= 160) && (circle[S1][S2] <= 223))
		{
			POS++;
			
			/* Move to */		
			amove[2]  = emove[X+S1];
			amove[3]  = emove[Y+S2];
								
			mvwprintw(winE,L1,14,"[%c%c/%c%c] %3d",
			amove[0],amove[1],amove[2],amove[3],circle[S1][S2]);
			
			/* Wait Time */
			usleep(usecs);
			
			/* Windows Refresh */
			wrefresh(winF);
			wrefresh(winE);
		}
	}
}

//-----------------------------------------------------------------------------

/* King */
void Game::king(int *ES,int F1,int F2,int T1,int T2)
{
	if(((T1 >= -1) && (T1 <= 1)) && ((T2 >= -1) && (T2 <= 1)) && ((T1 != 0) || (T2 != 0)))
	{
		engine(ES,F1,F2,T1,T2);
	}
}

//-----------------------------------------------------------------------------

/* Queen */
void Game::queen(int *ES,int F1,int F2,int T1,int T2)
{
	if((((T1 == 0) || (T2 == 0)) && ((T1 != 0) || (T2 != 0))))
	{
		engine(ES,F1,F2,T1,T2);
	}
	if((((T1 == T2) || ((T1 + T2) == 0))) && ((T1 != 0) || (T2 != 0)))
	{
		engine(ES,F1,F2,T1,T2);
	}
}

//-----------------------------------------------------------------------------

/* Turm */
void Game::turm(int *ES,int F1,int F2,int T1,int T2)
{
	if((((T1 == 0) || (T2 == 0)) && ((T1 != 0) || (T2 != 0))))
	{
		engine(ES,F1,F2,T1,T2);
	}
}

//-----------------------------------------------------------------------------

/* Bishop */
void Game::bishop(int *ES,int F1,int F2,int T1,int T2)
{
	if((((T1 == T2) || ((T1 + T2) == 0))) && ((T1 != 0) || (T2 != 0)))
	{
		engine(ES,F1,F2,T1,T2);
	}
}

//-----------------------------------------------------------------------------

/* Springer */
void Game::springer(int *ES,int F1,int F2,int T1,int T2)
{
	if((((T2 == -2) || (T2 == +2)) && ((T1 == -1) || (T1 == +1))))
	{
		engine(ES,F1,F2,T1,T2);
	}
	if((((T1 == -2) || (T1 == +2)) && ((T2 == -1) || (T2 == +1))))
	{
		engine(ES,F1,F2,T1,T2);
	}
}

//-----------------------------------------------------------------------------

/* People B */
void Game::people1(int *ES,int F1,int F2,int T1,int T2)
{
	if(((T1 == 0)) && (T2 == -2) && ((T1 != 0) || (T2 != 0)))
	{
		engine(ES,F1,F2,T1,T2);
	}
	if(((T1 >= -1) && (T1 <= +1)) && (T2 == -1) && ((T1 != 0) || (T2 != 0)))
	{
		engine(ES,F1,F2,T1,T2);
	}
}

//-----------------------------------------------------------------------------

/* People W */
void Game::people2(int *ES,int F1,int F2,int T1,int T2)
{	
	if(((T1 == 0)) && (T2 == +2) && ((T1 != 0) || (T2 != 0)))
	{
		engine(ES,F1,F2,T1,T2);
	}
	if(((T1 >= -1) && (T1 <= +1)) && (T2 == +1) && ((T1 != 0) || (T2 != 0)))
	{
		engine(ES,F1,F2,T1,T2);
	}
}

//-----------------------------------------------------------------------------

/* Squares */
void Game::squares(int *ES,int *sbit)
{
	int F1, F2, T1, T2;
	int D2, D3, D4;
	int G1, G2, G3, G4, G5, G6, G7, G8;
	
	/* White */
	cengine[5][1][1][C0] = '0';
	cengine[5][1][8][C0] = '+';
	
	cengine[8][1][5][C0] = 'r';
	cengine[7][1][3][C0] = 'n';
	cengine[6][1][4][C0] = 'b';
	cengine[5][1][7][C0] = 'k';
	cengine[4][1][6][C0] = 'q';
	cengine[3][1][4][C0] = 'b';
	cengine[2][1][3][C0] = 'n';
	cengine[1][1][5][C0] = 'r'; 
			
	cengine[8][2][2][C0] = 'p'; 
	cengine[7][2][2][C0] = 'p';
	cengine[6][2][2][C0] = 'p';
	cengine[5][2][2][C0] = 'p';
	cengine[4][2][2][C0] = 'p';
	cengine[3][2][2][C0] = 'p';
	cengine[2][2][2][C0] = 'p';
	cengine[1][2][2][C0] = 'p'; 
				
	/* Black */
	cengine[5][8][1][C1] = '0';
	cengine[5][8][8][C1] = '+';
	
	cengine[8][8][5][C1] = 'R';
	cengine[7][8][3][C1] = 'N';
	cengine[6][8][4][C1] = 'B';
	cengine[5][8][7][C1] = 'K';
	cengine[4][8][6][C1] = 'Q';
	cengine[3][8][4][C1] = 'B';
	cengine[2][8][3][C1] = 'N';
	cengine[1][8][5][C1] = 'R'; 
			
	cengine[8][7][2][C1] = 'P'; 
	cengine[7][7][2][C1] = 'P';
	cengine[6][7][2][C1] = 'P';
	cengine[5][7][2][C1] = 'P';
	cengine[4][7][2][C1] = 'P';
	cengine[3][7][2][C1] = 'P';
	cengine[2][7][2][C1] = 'P';
	cengine[1][7][2][C1] = 'P';

	//--------------------------------------------------------------------
	
	/* Matrix */
	matrix[8][C0][C0][C0] = '1';
	matrix[8][C0][C0][C1] = '1';
	matrix[8][C0][C1][C0] = '2';
	matrix[8][C0][C1][C1] = '2';
	matrix[8][C1][C0][C0] = '3';
	matrix[8][C1][C0][C1] = '3';
	matrix[8][C1][C1][C0] = '4'; 
	matrix[8][C1][C1][C1] = '4';

	//--------------------------------------------------------------------

	/* Array */
	D2 = ES[2];
	D3 = ES[3];
	D4 = ES[4];
	
	/* Result */
	G1 = sbit[0];
	G2 = sbit[1];
	G3 = sbit[2];
	G4 = sbit[3];
	G5 = sbit[4];
	G6 = sbit[5];
	G7 = sbit[6];
	G8 = sbit[7];
	
	//--------------------------------------------------------------------

	/* 64 Squares */
	for(F2 = C8; F2 >= C1; F2--)
	{
	for(F1 = C1; F1 <= C8; F1++)
	{
	/* Circle */
	if((circle[F1][F2] >= 160) && (circle[F1][F2] <= 223))
	{
	
	/* Formel */	
	formel(F1,F2);
	/* Position */
	POS = 0;
					
	/* Possibility */
	for(T2 = D; T2 <= C; T2++)
	{
	for(T1 = C; T1 >= D; T1--)
	{
		/* Chess Diplomatic */
		if(cengine[F1][F2][8][D4] && matrix[G8][D2][D3][D4])
		{	
			mvwprintw(winE,L1,12,"%c",cengine[F1][F2][8][D4]);
			king(ES,F1,F2,T1,T2);
		}
		/* Rochade */
		if(cengine[F1][F2][1][D4] && matrix[G1][D2][D3][D4])
		{	
			mvwprintw(winE,L1,12,"%c",cengine[F1][F2][1][D4]);
			king(ES,F1,F2,T1,T2);
		}
		/* K:  T0 | T0  */
		if(cengine[F1][F2][7][D4] && matrix[G7][D2][D3][D4])
		{	
			mvwprintw(winE,L1,12,"%c",cengine[F1][F2][7][D4]);
			king(ES,F1,F2,T1,T2);
		}
		/* Q:  T2 | T0  */
		if(cengine[F1][F2][6][D4] && matrix[G6][D2][D3][D4])
		{
			mvwprintw(winE,L1,12,"%c",cengine[F1][F2][6][D4]);
			queen(ES,F1,F2,T1,T2);
		}	
		/* R:  T2 | T0   */
		if(cengine[F1][F2][5][D4] && matrix[G5][D2][D3][D4])
		{
			mvwprintw(winE,L1,12,"%c",cengine[F1][F2][5][D4]);
			turm(ES,F1,F2,T1,T2);	
		}
		/* B:  T2 | T2   */	
		if(cengine[F1][F2][4][D4] && matrix[G4][D2][D3][D4])
		{
			mvwprintw(winE,L1,12,"%c",cengine[F1][F2][4][D4]);
			bishop(ES,F1,F2,T1,T2);
		}
		/* N:  T2 | T1   */
		if(cengine[F1][F2][3][D4] && matrix[G3][D2][D3][D4])
		{	
			mvwprintw(winE,L1,12,"%c",cengine[F1][F2][3][D4]);
			springer(ES,F1,F2,T1,T2);
		}
		/* P:  T1 | T0   */
		if(cengine[F1][F2][2][D4] && matrix[G2][D2][D3][D4])
		{		
			mvwprintw(winE,L1,12,"%c",cengine[F1][F2][2][D4]);
			if(D4 == 0)
			{
				people2(ES,F1,F2,T1,T2);
			}
			if(D4 == 1)
			{
				people1(ES,F1,F2,T1,T2);
			}
		}
	}}}}}
}

//-----------------------------------------------------------------------------

/* Sort */
void Game::sort(char *nmove)
{
	int FIGU, FELD, L2;
	int R1, R2, R3, R4;
	int E1, E2, E3, E4; 
	int ES[C8];
	
	/* Window Box */
	winF = newwin(11,76,13,2);
	winE = newwin(9,74,14,3);
	box(winF,ACS_VLINE,ACS_HLINE);
	scrollok(winE,true);
	
	/* Windows Refresh */
	touchwin(stdscr);
	refresh();
	wrefresh(winF);
	wrefresh(winE);

	/* Lines */
	L1 = -1;
	
	//------------------------------------------------------------------------
	
	/* Engine Deep */	
	for(FIGU = -8, FELD = 0; FIGU < 0; FIGU += 2)
	{
	/* Tree */
	for(E1 = C0; E1 <= C1; E1++)
	{
	
	/* Lines */
	L1++; 
	L2 = 22;
	R1 = -1; 
	wprintw(winE,"\n"); 
	
	/* Notation */
	for(E2 = C0; E2 <= C1; E2++)
	{
	/* Direction */
	for(E3 = C0; E3 <= C1; E3++)
	{	
	/* Material */
	for(E4 = C0; E4 <= C1; E4++)
	{	
		/* Counter */
		if(L1 >= 8)
		{
			L1 = 8;
		}
		
		/* Array */
		ES[1] = E1;
		ES[2] = E2;
		ES[3] = E3;
		ES[4] = E4;
		
		/* Engine Info */
		FELD = FELD | FIGU;
		mvwprintw(winE,L1,0,"%+3d %d%d%d%d ", 
		FELD, ES[1]+3, ES[2]+5, ES[3], ES[4]+1);
		
		//--------------------------------------------------------------------	
		
		/* Selection Sort */
		R1++; 
		R3 = R1;

		for(R2 = R1 + 1; R2 <= C8; R2++)
		{	
			if(sbit[R2] > sbit[R3])
			{
				R3 = R2;
			}
		}
		if(R3 != R1)
		{
			R4 = sbit[R3];
			sbit[R3] = sbit[R1];
			sbit[R1] = R4;
		}

		//--------------------------------------------------------------------	
				
		/* Insert Sort */
		R4 = sbit[L1];
		
		for(R2 = L1; sbit[R2-1] > R4 && R2 > C0; R2--)
		{	
			sbit[R2] = sbit[R2-1];
		}	
		sbit[R2] = R4;
		
		//--------------------------------------------------------------------
		
		/* 8 / 4 / 2 / 1 */
		if((E1 == 0) || (E1 == 1))
		{
			if((E2 == 0) && (E3 == 0))
			{
				/* Function */
				squares(ES,sbit);
			}
			if((E2 == 0) && (E3 == 1))
			{
				/* Function */
				squares(ES,sbit);
			} 
			if((E2 == 1) && (E3 == 0))
			{	
				/* Function */
				squares(ES,sbit);
			}
			if((E2 == 1) && (E3 == 1))
			{
				/* Function */
				squares(ES,sbit);
			}
		}
		
		//--------------------------------------------------------------------

		L2 += 5;

		/* Move Notation */
		mvwprintw(winE,L1,L2,"%c%c%c%c",amove[0],amove[1],amove[2],amove[3]);
	
		/* Control Sort */
		//mvwprintw(winE,L1,L2,"%d%d%d%d",sbit[0],sbit[1],sbit[2],sbit[3]);

		/* Engine to Board */
		nmove[0] = amove[0];
		nmove[1] = amove[1];
		nmove[2] = amove[2];
		nmove[3] = amove[3];
		
		/* Windows Refresh */
		wrefresh(winF);
		wrefresh(winE);
		
	}}}}}
}

//-----------------------------------------------------------------------------
